/*
 Wireless Gameboy Controller TX with RX Game Test
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 23/09/2018
 Last Modified: 5/11/2018
 
 Program the ATmega48PA to use an external crystal (16MHz) and divide clock by 1
 avrdude -p ATmega48p -c usbasp -U lfuse:w:0xEF:m -U hfuse:w:0xDF:m -U efuse:w:0xFF:m
 
 You will need to cut the A15 line going to the flash chip CE line and use a DPDT switch to have the CE line select between
 A15 (when programming) and A14 (when running on a Gameboy)
 
*/

// ATmega48P Pin map
// -------------------------------------
// PB0 - LED			PC0 - A12		PD0 - D0
// PB1 - nRF CE		PC1 - A13		PD1 - D1
// PB2 - nRF CSN		PC2 - A14		PD2 - D2
// PB3 - nRF MOSI		PC3 - A15		PD3 - D3
// PB4 - nRF MISO		PC4 - WR			PD4 - D4
// PB5 - nRF SCK							PD5 - D5
//												PD6 - D6
//												PD7 - D7


#define F_CPU 16000000 // 16 MHz

#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "setup.c"
#include "mirf.c"


int main(void) {
	setup();
	
	uint8_t bytesToSend = 0;
	uint8_t sendCounter = 0;
	uint8_t bytesToReceive = 0;
	uint8_t receiveCounter = 0;
	uint8_t stage = 0;
	uint8_t nextByteWrite = 0;
	
	dataIn[0] = 100; // x
	dataIn[1] = 30; // y
	dataInCount = 4;
	
	while(1) {
		if (stage == 0) {
			mirf_CE_lo; // Stop listening
			if (mirf_status() & (1<<RX_DR)) { 
				mirf_CSN_lo; // Pull down chip select
				spi_transfer(R_RX_PAYLOAD); // Send cmd to read rx payload
				spi_read_data(dataIn, mirf_PAYLOAD); // Read payload
				
				mirf_CSN_hi; // Pull up chip select
				mirf_config_register(STATUS,(1<<RX_DR)); // Reset status register
				
				mirf_flush_rx_tx(); // Flush TX/RX
			}
			
			mirf_transmit_data(4);
			
			NRF_RX_POWERUP;
			_delay_us(130);
			mirf_flush_rx_tx(); // Flush TX/RX
			mirf_CE_hi; // Start listening
		}
		
		// Wait for WR to go low
		while (PINC & 0x10);
		uint8_t readAddr = PINC; // Read address
		asm volatile("nop");
		uint8_t readData = PIND; // Read data
		
		if ((readAddr & 0x1F) == 0x07) { // Address 0x7000 and WR low
			if (stage == 0) { // Read the number of bytes we will receive from the GB to send to the nRF
				bytesToSend = readData;
				sendCounter = 0;
				receiveCounter = 0;
				stage++;
			}
			else if (stage == 1) { // Store the bytes now
				if (sendCounter < bytesToSend) {
					dataOut[sendCounter] = readData;
					sendCounter++;
				}
				if (sendCounter == bytesToSend) { // Finished, next stage
					stage++;
				}
			}
			else if (stage == 2) { // Read the number of bytes we want to receive from the nRF
				bytesToReceive = readData;
				
				if (dataInCount == bytesToReceive) { // Has to match the payload size for now
					stage++;
				}
				else {
					bytesToReceive = 0;
					stage = 0;
				}
				
				while (!(PINC & 0x08)); // Wait for A15 to go high
				asm volatile("nop"); // NOP a couple of times until the GB is ready to receive the data
				asm volatile("nop");
				
				PORTD = bytesToReceive; // Change to the data
				DDRD = 0xFF; // All pins as outputs
				PORTB |= (1<<LED);
				asm volatile("nop");
				
				// Change back to input
				DDRD = 0x00;
				PORTD = 0x00;
				PORTB &= ~(1<<LED);
				
				// Preload the first byte to write
				nextByteWrite = dataIn[receiveCounter];
			}
			else if (stage == 3) { // Write the bytes to the Gameboy now
				if (receiveCounter < bytesToReceive) {
					while (!(PINC & 0x08)); // Wait for A15 to go high
					asm volatile("nop"); // NOP a couple of times until the GB is ready to receive the data
					asm volatile("nop");
					
					// Change to the data and set as output
					PORTD = nextByteWrite;
					DDRD = 0xFF;
					PORTB |= (1<<LED);
					asm volatile("nop");
					
					// Change back to input
					DDRD = 0x00;
					PORTD = 0x00;
					PORTB &= ~(1<<LED);
					
					receiveCounter++;
					nextByteWrite = dataIn[receiveCounter];
				}
				if (receiveCounter == bytesToReceive) {
					stage = 0;
				}
			}
		}
	}
}
